/********************************** (C) COPYRIGHT *******************************
 * File Name          : app_usb.c
 * Author             : WCH
 * Version            : V1.1
 * Date               : 2022/01/19
 * Description        :
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

 /*********************************************************************
  * INCLUDES
  */

#include "CONFIG.h"
#include "gattprofile.h"
#include "stdint.h"
#include "ble_usb_service.h"
#include "app_usb.h"
#include "peripheral.h"
#include "RingMem.h"

  /*********************************************************************
   * MACROS
   */

   /*  */
   //  3.1 Requests---Abstract Control Model
#define DEF_SEND_ENCAPSULATED_COMMAND  0x00
#define DEF_GET_ENCAPSULATED_RESPONSE  0x01
#define DEF_SET_COMM_FEATURE           0x02
#define DEF_GET_COMM_FEATURE           0x03
#define DEF_CLEAR_COMM_FEATURE         0x04
#define DEF_SET_LINE_CODING          0x20   // Configures DTE rate, stop-bits, parity, and number-of-character
#define DEF_GET_LINE_CODING          0x21   // This request allows the host to find out the currently configured line coding.
//#define DEF_SET_CTL_LINE_STE         0X22   // This request generates RS-232/V.24 style control signals.
#define DEF_SET_CONTROL_LINE_STATE     0x22
#define DEF_SEND_BREAK                 0x23

//  3.2 Notifications---Abstract Control Model
#define DEF_NETWORK_CONNECTION         0x00
#define DEF_RESPONSE_AVAILABLE         0x01
#define DEF_SERIAL_STATE               0x20

/* Character Size */
#define HAL_UART_5_BITS_PER_CHAR             5
#define HAL_UART_6_BITS_PER_CHAR             6
#define HAL_UART_7_BITS_PER_CHAR             7
#define HAL_UART_8_BITS_PER_CHAR             8

/* Stop Bits */
#define HAL_UART_ONE_STOP_BIT                1
#define HAL_UART_TWO_STOP_BITS               2

/* Parity settings */
#define HAL_UART_NO_PARITY                   0x00                              //У
#define HAL_UART_ODD_PARITY                  0x01                              //У
#define HAL_UART_EVEN_PARITY                 0x02                              //żУ
#define HAL_UART_MARK_PARITY                 0x03                              //1 mark
#define HAL_UART_SPACE_PARITY                0x04                              //հλ space

//Line Codeṹ
typedef struct __PACKED _LINE_CODE
{
	UINT32  BaudRate;   /*  */
	UINT8 StopBits;   /* ֹͣλ01ֹͣλ11.5ֹͣλ22ֹͣλ */
	UINT8 ParityType;   /* Уλ0None1Odd2Even3Mark4Space */
	UINT8 DataBits;   /* λ567816 */
}LINE_CODE, * PLINE_CODE;

   /*********************************************************************
	* CONSTANTS
	*/
uint8_t DevConfig, Ready;
uint8_t SetupReqCode;
UINT16 SetupReqLen;
const uint8_t* pDescr;

#define DevEP0SIZE  0x40
// 豸
const uint8_t MyDevDescr[] = { 
	0x12,                   //bLength(18);
	0x01,                   //bDescriptorType(Device);
	0x00,0x02,              //bcdUSB(2.00);
	0xef,                   //bDeviceClass(Miscellaneous Device Class);
	0x02,                   //bDeviceSubClass(Common Class);
	0x01,                   //bDeviceProtocol(Interface Association Descriptor);
	0x40,                   //bMaxPacketSize0(64);
	0xbf,0x43,              //idVendor(34bf);
	0x0a,0x99,              //idProduct(ff0a);
	0x00,0x01,              //bcdDevice(1.00);
	0x01,                   //iManufacturer(1);
	0x02,                   //iProduct(2);
	0x00,                   //iSerialNumber(0);
	0x01,                   //bNumConfigurations(1);
};
// 
const uint8_t MyCfgDescr[] = { 
	// 
	0x09,                   //bLength(9);
	0x02,                   //bDescriptorType(Configuration);
	0xCF,0x00,              //wTotalLength(141);
	0x06,                   //bNumInterfaces(6);
	0x01,                   //bConfigurationValue(1);
	0x00,                   //iConfiguration(0);
	0x80,                   //bmAttributes(BUSPower);
	0x32,                   //MaxPower(100mA);

	// IAD
	0x08,                   //bLength(8);
	0x0b,                   //bDescriptorType(IAD);
	0x00,                   //bFirstInterface(0);
	0x02,                   //bInterfaceCount(2);
	0x02,                   //bFunctionClass(Communication Device Class);
	0x02,                   //bInterfaceSubClass(Abstract Control Model);
	0x01,                   //bInterfaceProtocol(Common AT commands);
	0x00,                   //iInterface(0);

		// ---------------- Interface Descriptor -----------------
		0x09,                   //bLength(9);
		0x04,                   //bDescriptorType(Interface);
		0x00,                   //bInterfaceNumber(0);
		0x00,                   //bAlternateSetting(0);
		0x01,                   //bNumEndpoints(1);
		0x02,                   //bInterfaceClass(Communication Interface Class);
		0x02,                   //bInterfaceSubClass(Abstract Control Model);
		0x01,                   //bInterfaceProtocol(Common AT commands);
		0x00,                   //iInterface(0);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x00,                   //bDescriptorSubtype(Header Functional Descriptor);
			0x10,0x01,              //bcdCDC(1.10);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x01,                   //bDescriptorSubtype(Call Management Functional Descriptor);
			0x00,                   //bmCapabilities(Device does not handles call management itself);
			0x01,                   //bDataInterface(1);

			0x04,                   //bLength(4);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x02,                   //bDescriptorSubtype(Abstract Control Management Functional Descriptor);
			0x02,                   //bmCapabilities(Set/Get_Line_Coding,Serial_State,Set_Control_Line_State);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x06,                   //bDescriptorSubtype(Union Functional descriptor);
			0x00,                   //bMasterInterface(0);
			0x01,                   //bSlaveInterface0(1);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x81,                   //bEndpointAddress(EndPoint1 as IN);
			0x03,                   //bmAttributes(Interrupt);
			0x40,0x00,              //wMaxPacketSize(64);
			0xff,                   //bInterval(255ms);

		// ---------------- Interface Descriptor -----------------
		0x09,                   //bLength(9);
		0x04,                   //bDescriptorType(Interface);
		0x01,                   //bInterfaceNumber(1);
		0x00,                   //bAlternateSetting(0);
		0x02,                   //bNumEndpoints(2);
		0x0a,                   //bInterfaceClass(Data Interface Class);
		0x00,                   //bInterfaceSubClass(AData Interface Class SubClass Codes);
		0x00,                   //bInterfaceProtocol(USB SPEC);
		0x00,                   //iInterface(0);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x82,                   //bEndpointAddress(EndPoint2 as IN);
			0x02,                   //bmAttributes(Bulk);
			0x40,0x00,              //wMaxPacketSize(64);
			0x00,                   //bInterval(Ignored);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x02,                   //bEndpointAddress(EndPoint2 as OUT);
			0x02,                   //bmAttributes(Bulk);
			0x40,0x00,              //wMaxPacketSize(64);
			0x00,                   //bInterval(Ignored);

	0x08,                   //bLength(8);
	0x0b,                   //bDescriptorType(IAD);
	0x02,                   //bFirstInterface(2);
	0x02,                   //bInterfaceCount(2);
	0x02,                   //bFunctionClass(Communication Device Class);
	0x02,                   //bInterfaceSubClass(Abstract Control Model);
	0x01,                   //bInterfaceProtocol(Common AT commands);
	0x00,                   //iInterface(0);

		// ---------------- Interface Descriptor -----------------
		0x09,                   //bLength(9);
		0x04,                   //bDescriptorType(Interface);
		0x02,                   //bInterfaceNumber(2);
		0x00,                   //bAlternateSetting(0);
		0x01,                   //bNumEndpoints(1);
		0x02,                   //bInterfaceClass(Communication Interface Class);
		0x02,                   //bInterfaceSubClass(Abstract Control Model);
		0x01,                   //bInterfaceProtocol(Common AT commands);
		0x00,                   //iInterface(0);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x00,                   //bDescriptorSubtype(Header Functional Descriptor);
			0x10,0x01,              //bcdCDC(1.10);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x01,                   //bDescriptorSubtype(Call Management Functional Descriptor);
			0x00,                   //bmCapabilities(Device does not handles call management itself);
			0x01,                   //bDataInterface(1);

			0x04,                   //bLength(4);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x02,                   //bDescriptorSubtype(Abstract Control Management Functional Descriptor);
			0x02,                   //bmCapabilities(Set/Get_Line_Coding,Serial_State,Set_Control_Line_State);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x06,                   //bDescriptorSubtype(Union Functional descriptor);
			0x00,                   //bMasterInterface(0);
			0x01,                   //bSlaveInterface0(1);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x83,                   //bEndpointAddress(EndPoint3 as IN);
			0x03,                   //bmAttributes(Interrupt);
			0x40,0x00,              //wMaxPacketSize(64);
			0xff,                   //bInterval(255ms);

		// ---------------- Interface Descriptor -----------------
		0x09,                   //bLength(9);
		0x04,                   //bDescriptorType(Interface);
		0x03,                   //bInterfaceNumber(3);
		0x00,                   //bAlternateSetting(0);
		0x02,                   //bNumEndpoints(2);
		0x0a,                   //bInterfaceClass(Data Interface Class);
		0x00,                   //bInterfaceSubClass(AData Interface Class SubClass Codes);
		0x00,                   //bInterfaceProtocol(USB SPEC);
		0x00,                   //iInterface(0);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x84,                   //bEndpointAddress(EndPoint4 as IN);
			0x02,                   //bmAttributes(Bulk);
			0x40,0x00,              //wMaxPacketSize(64);
			0x00,                   //bInterval(Ignored);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x04,                   //bEndpointAddress(EndPoint4 as OUT);
			0x02,                   //bmAttributes(Bulk);
			0x40,0x00,              //wMaxPacketSize(64);
			0x00,                   //bInterval(Ignored);

	0x08,                   //bLength(8);
	0x0b,                   //bDescriptorType(IAD);
	0x04,                   //bFirstInterface(4);
	0x02,                   //bInterfaceCount(2);
	0x02,                   //bFunctionClass(Communication Device Class);
	0x02,                   //bInterfaceSubClass(Abstract Control Model);
	0x01,                   //bInterfaceProtocol(Common AT commands);
	0x00,                   //iInterface(0);
	
		// ---------------- Interface Descriptor -----------------
		0x09,                   //bLength(9);
		0x04,                   //bDescriptorType(Interface);
		0x04,                   //bInterfaceNumber(4);
		0x00,                   //bAlternateSetting(0);
		0x01,                   //bNumEndpoints(1);
		0x02,                   //bInterfaceClass(Communication Interface Class);
		0x02,                   //bInterfaceSubClass(Abstract Control Model);
		0x01,                   //bInterfaceProtocol(Common AT commands);
		0x00,                   //iInterface(0);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x00,                   //bDescriptorSubtype(Header Functional Descriptor);
			0x10,0x01,              //bcdCDC(1.10);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x01,                   //bDescriptorSubtype(Call Management Functional Descriptor);
			0x00,                   //bmCapabilities(Device does not handles call management itself);
			0x01,                   //bDataInterface(1);

			0x04,                   //bLength(4);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x02,                   //bDescriptorSubtype(Abstract Control Management Functional Descriptor);
			0x02,                   //bmCapabilities(Set/Get_Line_Coding,Serial_State,Set_Control_Line_State);

			0x05,                   //bLength(5);
			0x24,                   //bDescriptorType(CS_INTERFACE);
			0x06,                   //bDescriptorSubtype(Union Functional descriptor);
			0x00,                   //bMasterInterface(0);
			0x01,                   //bSlaveInterface0(1);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x85,                   //bEndpointAddress(EndPoint5 as IN);
			0x03,                   //bmAttributes(Interrupt);
			0x40,0x00,              //wMaxPacketSize(64);
			0xff,                   //bInterval(255ms);

		// ---------------- Interface Descriptor -----------------
		0x09,                   //bLength(9);
		0x04,                   //bDescriptorType(Interface);
		0x05,                   //bInterfaceNumber(5);
		0x00,                   //bAlternateSetting(0);
		0x02,                   //bNumEndpoints(2);
		0x0a,                   //bInterfaceClass(Data Interface Class);
		0x00,                   //bInterfaceSubClass(AData Interface Class SubClass Codes);
		0x00,                   //bInterfaceProtocol(USB SPEC);
		0x00,                   //iInterface(0);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x86,                   //bEndpointAddress(EndPoint6 as IN);
			0x02,                   //bmAttributes(Bulk);
			0x40,0x00,              //wMaxPacketSize(64);
			0x00,                   //bInterval(Ignored);

			0x07,                   //bLength(7);
			0x05,                   //bDescriptorType(Endpoint);
			0x06,                   //bEndpointAddress(EndPoint6 as OUT);
			0x02,                   //bmAttributes(Bulk);
			0x40,0x00,              //wMaxPacketSize(64);
			0x00,                   //bInterval(Ignored);
};                       
// 
const uint8_t MyLangDescr[] = { 0x04, 0x03, 0x09, 0x04 };
// Ϣ
const uint8_t MyManuInfo[] = { 0x0E, 0x03, 'w', 0, 'c', 0, 'h', 0, '.', 0, 'c', 0, 'n', 0 };
// ƷϢ
const uint8_t MyProdInfo[] = { 0x0C, 0x03, 'C', 0, 'H', 0, '5', 0, '8', 0, 'x', 0 };
/*Ʒ*/
const uint8_t StrDesc[28] =
{
  0x1C,0x03,0x55,0x00,0x53,0x00,0x42,0x00,
  0x32,0x00,0x2E,0x00,0x30,0x00,0x2D,0x00,
  0x53,0x00,0x65,0x00,0x72,0x00,0x69,0x00,
  0x61,0x00,0x6C,0x00
};

const uint8_t Return1[2] = { 0x31,0x00 };
const uint8_t Return2[2] = { 0xC3,0x00 };
const uint8_t Return3[2] = { 0x9F,0xEE };



/*********************************************************************
 * LOCAL VARIABLES
 */

 /******** ûԶ˵RAM ****************************************/
__attribute__((aligned(4)))  uint8_t EP0_Databuf[64 + 64 + 64];    //ep0(64)+ep4_out(64)+ep4_in(64)
__attribute__((aligned(4)))  uint8_t EP1_Databuf[64 + 64];    //ep1_out(64)+ep1_in(64)
__attribute__((aligned(4)))  uint8_t EP2_Databuf[64 + 64];    //ep2_out(64)+ep2_in(64)
__attribute__((aligned(4)))  uint8_t EP3_Databuf[64 + 64];    //ep3_out(64)+ep3_in(64)

__attribute__((aligned(4)))  uint8_t EP5_Databuf[64 + 64];    //ep3_out(64)+ep3_in(64)
__attribute__((aligned(4)))  uint8_t EP6_Databuf[64 + 64];    //ep3_out(64)+ep3_in(64)
__attribute__((aligned(4)))  uint8_t EP7_Databuf[64 + 64];    //ep3_out(64)+ep3_in(64)



/*  */
LINE_CODE Uart0Para;

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

 /*********************************************************************
  * @fn      app_usb_init
  *
  * @brief   ʼusb
  *
  * @return  none
  */
void app_usb_init()
{
	pEP0_RAM_Addr = EP0_Databuf;
	pEP1_RAM_Addr = EP1_Databuf;
	pEP2_RAM_Addr = EP2_Databuf;
	pEP3_RAM_Addr = EP3_Databuf;
	pEP5_RAM_Addr = EP5_Databuf;
	pEP6_RAM_Addr = EP3_Databuf;
	pEP7_RAM_Addr = EP7_Databuf;


	Uart0Para.BaudRate = 115200;
	Uart0Para.DataBits = HAL_UART_8_BITS_PER_CHAR;
	Uart0Para.ParityType = HAL_UART_NO_PARITY;
	Uart0Para.StopBits = HAL_UART_ONE_STOP_BIT;

	USB_DeviceInit();
	PFIC_EnableIRQ(USB_IRQn);
}

/*********************************************************************
 * @fn      USBSendData
 *
 * @brief   ݸ
 *
 * @return  none
 */
uint8_t USBSendData(void)
{
	if (R8_UEP2_CTRL & UEP_T_RES_NAK == UEP_T_RES_NAK)
	{
		return FAILURE;
	}
	if (RingMemBLE.CurrentLen > 32)
	{
		RingMemRead(&RingMemBLE, pEP2_IN_DataBuf, 32);
		DevEP2_IN_Deal(32);
	}
	else
	{
		uint8_t len = RingMemBLE.CurrentLen;
		RingMemRead(&RingMemBLE, pEP2_IN_DataBuf, len);
		DevEP2_IN_Deal(len);
	}
	return SUCCESS;
}

/*********************************************************************
 * @fn      DevEP1_OUT_Deal
 *
 * @brief   ˵1ݴ
 *
 * @return  none
 */
void DevEP1_OUT_Deal(uint8_t l)
{ /* ûԶ */
}

/*********************************************************************
 * @fn      DevEP2_OUT_Deal
 *
 * @brief   ˵2ݴ
 *
 * @return  none
 */
void DevEP2_OUT_Deal(uint8_t l)
{
	memcpy(pEP2_IN_DataBuf, pEP2_OUT_DataBuf, l);
	DevEP2_IN_Deal(l);
}

/*********************************************************************
 * @fn      DevEP3_OUT_Deal
 *
 * @brief   ˵3ݴ
 *
 * @return  none
 */
void DevEP3_OUT_Deal(uint8_t l)
{ /* ûԶ */
}

/*********************************************************************
 * @fn      DevEP4_OUT_Deal
 *
 * @brief   ˵4ݴ
 *
 * @return  none
 */
void DevEP4_OUT_Deal(uint8_t l)
{ 
	memcpy(pEP4_IN_DataBuf, pEP4_OUT_DataBuf, l);
	DevEP4_IN_Deal(l);
}

/*********************************************************************
 * @fn      DevEP5_OUT_Deal
 *
 * @brief   ˵5ݴ
 *
 * @return  none
 */
void DevEP5_OUT_Deal(uint8_t l)
{ /* ûԶ */
}

/*********************************************************************
 * @fn      DevEP6_OUT_Deal
 *
 * @brief   ˵6ݴ
 *
 * @return  none
 */
void DevEP6_OUT_Deal(uint8_t l)
{ 
	memcpy(pEP6_IN_DataBuf, pEP6_OUT_DataBuf, l);
	DevEP6_IN_Deal(l);
}

/*********************************************************************
 * @fn      DevEP7_OUT_Deal
 *
 * @brief   ˵7ݴ
 *
 * @return  none
 */
void DevEP7_OUT_Deal(uint8_t l)
{ /* ûԶ */
}



/*********************************************************************
 * @fn      USB_DevTransProcess
 *
 * @brief   USB 䴦
 *
 * @return  none
 */
void USB_DevTransProcess(void)
{
	uint8_t len, chtype;
	uint8_t intflag, errflag = 0;
	UINT8   data_dir = 0;   //ݷ

	intflag = R8_USB_INT_FG;
	if (intflag & RB_UIF_TRANSFER)
	{
		if ((R8_USB_INT_ST & MASK_UIS_TOKEN) != MASK_UIS_TOKEN)    // ǿ
		{
			switch (R8_USB_INT_ST & (MASK_UIS_TOKEN | MASK_UIS_ENDP))
				// ƺͶ˵
			{
			case UIS_TOKEN_IN | 0:
				{
					switch (SetupReqCode)
					{
					case USB_GET_DESCRIPTOR:
						len = SetupReqLen >= DevEP0SIZE ?
							DevEP0SIZE : SetupReqLen;    // δ䳤
						memcpy(pEP0_DataBuf, pDescr, len); /* ϴ */
						SetupReqLen -= len;
						pDescr += len;
						R8_UEP0_T_LEN = len;
						R8_UEP0_CTRL ^= RB_UEP_T_TOG;                             // ת
						break;
					case USB_SET_ADDRESS:
						R8_USB_DEV_AD = (R8_USB_DEV_AD & RB_UDA_GP_BIT) | SetupReqLen;
						R8_UEP0_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK;
						break;
					default:
						R8_UEP0_T_LEN = 0;                                      // ״̬׶жϻǿϴ0ݰƴ
						R8_UEP0_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK;
						break;
					}
				}
				break;

			case UIS_TOKEN_OUT | 0:
				{
					len = R8_USB_RX_LEN;
				}
				break;

			case UIS_TOKEN_OUT | 1:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{                       // ͬݰ
						len = R8_USB_RX_LEN;
						DevEP1_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 1:
				R8_UEP1_CTRL = (R8_UEP1_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			case UIS_TOKEN_OUT | 2:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{                       // ͬݰ
						len = R8_USB_RX_LEN;
						DevEP2_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 2:
				R8_UEP2_CTRL = (R8_UEP2_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			case UIS_TOKEN_OUT | 3:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{                       // ͬݰ
						len = R8_USB_RX_LEN;
						DevEP3_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 3:
				R8_UEP3_CTRL = (R8_UEP3_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			case UIS_TOKEN_OUT | 4:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{
						R8_UEP4_CTRL ^= RB_UEP_R_TOG;
						len = R8_USB_RX_LEN;
						DevEP4_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 4:
				R8_UEP4_CTRL ^= RB_UEP_T_TOG;
				R8_UEP4_CTRL = (R8_UEP4_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			case UIS_TOKEN_OUT | 5:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{                       // ͬݰ
						len = R8_USB_RX_LEN;
						DevEP5_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 5:
				R8_UEP5_CTRL = (R8_UEP5_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			case UIS_TOKEN_OUT | 6:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{                       // ͬݰ
						len = R8_USB_RX_LEN;
						DevEP6_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 6:
				R8_UEP6_CTRL = (R8_UEP6_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			case UIS_TOKEN_OUT | 7:
				{
					if (R8_USB_INT_ST & RB_UIS_TOG_OK)
					{                       // ͬݰ
						len = R8_USB_RX_LEN;
						DevEP7_OUT_Deal(len);
					}
				}
				break;

			case UIS_TOKEN_IN | 7:
				R8_UEP7_CTRL = (R8_UEP7_CTRL & ~MASK_UEP_T_RES) | UEP_T_RES_NAK;
				break;

			default:
				break;
			}
			R8_USB_INT_FG = RB_UIF_TRANSFER;
		}
		if (R8_USB_INT_ST & RB_UIS_SETUP_ACT)/* Setup */
		{
			R8_UEP0_CTRL = RB_UEP_R_TOG | RB_UEP_T_TOG | UEP_R_RES_ACK | UEP_T_RES_NAK;
			SetupReqLen  = pSetupReqPak->wLength;
			SetupReqCode = pSetupReqPak->bRequest;
			chtype       = pSetupReqPak->bRequestType;
			len          = 0;
			errflag      = 0;

			/* ݷ */
			if (pSetupReqPak->bRequestType & USB_REQ_TYP_IN)
			{
				data_dir = USB_REQ_TYP_IN;
			}
			else
			{
				data_dir = USB_REQ_TYP_OUT;
			}

			if ((pSetupReqPak->bRequestType & USB_REQ_TYP_MASK) == USB_REQ_TYP_STANDARD) /* ׼ */
			{
				switch (SetupReqCode)
				{
				case USB_GET_DESCRIPTOR:
					{
						switch (((pSetupReqPak->wValue) >> 8))
						{
						case USB_DESCR_TYP_DEVICE:
							{
								pDescr = MyDevDescr;
								len = sizeof(MyDevDescr);
							}
							break;

						case USB_DESCR_TYP_CONFIG:
							{
								pDescr = MyCfgDescr;
								len = sizeof(MyCfgDescr);
							}
							break;

						case USB_DESCR_TYP_REPORT:
							//              {
							//                if ( ( ( pSetupReqPak->wIndex ) & 0xff ) == 0 )                             //ӿ0
							//                {
							//                  pDescr = KeyRepDesc;                                  //׼ϴ
							//                  len = sizeof( KeyRepDesc );
							//                }
							//                else if ( ( ( pSetupReqPak->wIndex ) & 0xff ) == 1 )                        //ӿ1
							//                {
							//                  pDescr = MouseRepDesc;                                //׼ϴ
							//                  len = sizeof( MouseRepDesc );
							//                  Ready = 1;                                            //иӿڣñ׼λӦһӿɺЧ
							//                }
							//                else
							//                  len = 0xff;                                           //ֻ2ӿڣ仰ִ
							//              }
							break;

						case USB_DESCR_TYP_STRING:
							{
								switch ((pSetupReqPak->wValue) & 0xff)
								{
								case 1:
									pDescr = MyManuInfo;
									len = MyManuInfo[0];
									break;
								case 2:
									pDescr = StrDesc;
									len = StrDesc[0];
									break;
								case 0:
									pDescr = MyLangDescr;
									len = MyLangDescr[0];
									break;
								default:
									errflag = 0xFF;                               // ֵַ֧
									break;
								}
							}
							break;

						default:
							errflag = 0xff;
							break;
						}
						if (SetupReqLen > len)
							SetupReqLen = len;      //ʵϴܳ
						len = (SetupReqLen >= DevEP0SIZE) ?
							DevEP0SIZE : SetupReqLen;
						memcpy(pEP0_DataBuf, pDescr, len);
						pDescr += len;
					}
					break;

				case USB_SET_ADDRESS:
					SetupReqLen = (pSetupReqPak->wValue) & 0xff;
					break;

				case USB_GET_CONFIGURATION:
					pEP0_DataBuf[0] = DevConfig;
					if (SetupReqLen > 1)
						SetupReqLen = 1;
					break;

				case USB_SET_CONFIGURATION:
					DevConfig = (pSetupReqPak->wValue) & 0xff;
					break;

				case USB_CLEAR_FEATURE:
					{
						if ((pSetupReqPak->bRequestType & USB_REQ_RECIP_MASK) == USB_REQ_RECIP_ENDP)    // ˵
						{
							switch ((pSetupReqPak->wIndex) & 0xff)
							{
							case 0x82:
								R8_UEP2_CTRL = (R8_UEP2_CTRL & ~(RB_UEP_T_TOG | MASK_UEP_T_RES)) | UEP_T_RES_NAK;
								break;
							case 0x02:
								R8_UEP2_CTRL = (R8_UEP2_CTRL & ~(RB_UEP_R_TOG | MASK_UEP_R_RES)) | UEP_R_RES_ACK;
								break;
							case 0x81:
								R8_UEP1_CTRL = (R8_UEP1_CTRL & ~(RB_UEP_T_TOG | MASK_UEP_T_RES)) | UEP_T_RES_NAK;
								break;
							case 0x01:
								R8_UEP1_CTRL = (R8_UEP1_CTRL & ~(RB_UEP_R_TOG | MASK_UEP_R_RES)) | UEP_R_RES_ACK;
								break;
							default:
								errflag = 0xFF;                                 // ֵ֧Ķ˵
								break;
							}
						}
						else
							errflag = 0xFF;
					}
					break;

				case USB_GET_INTERFACE:
					pEP0_DataBuf[0] = 0x00;
					if (SetupReqLen > 1)
						SetupReqLen = 1;
					break;

				case USB_GET_STATUS:
					pEP0_DataBuf[0] = 0x00;
					pEP0_DataBuf[1] = 0x00;
					if (SetupReqLen > 2)
						SetupReqLen = 2;
					break;

				default:
					errflag = 0xff;
					break;
				}
			}
			else if ((pSetupReqPak->bRequestType & USB_REQ_TYP_MASK) == USB_REQ_TYP_CLASS) /*  */
			{
				printf("    data_dir %d, SetupReqCode %x\r\n", data_dir, SetupReqCode);
				/* ´ */
				if (data_dir == USB_REQ_TYP_OUT)
				{
					switch (SetupReqCode)  // 
					{
					case DEF_SET_LINE_CODING: /* SET_LINE_CODING */
						{
							UINT8 i;
							printf("    SET_LINE_CODING\r\n    ");
							for (i = 0; i < 8; i++)
							{
								printf("%02x ", pEP0_DataBuf[i]);
							}
							printf("\r\n");
							if (pEP0_DataBuf[4] == 0x00)
							{
								//CDCSetSerIdx = 0;
								len = 0x00;
							}
							else if (pEP0_DataBuf[4] == 0x02)
							{
								//CDCSetSerIdx = 1;
								len = 0x00;
							}
							else if (pEP0_DataBuf[4] == 0x04)
							{
								//CDCSetSerIdx = 2;
								len = 0x00;
							}
							else
							{
								printf("Error %d\r\n", __LINE__);
								len = 0xFF;
							}
							break;
						}
					case DEF_SET_CONTROL_LINE_STATE:  /* SET_CONTROL_LINE_STATE */
						{
							UINT8  carrier_sta;
							UINT8  present_sta;
							/* ·״̬ */
							printf("    ctl %02x %02x\r\n", pEP0_DataBuf[2], pEP0_DataBuf[3]);
							carrier_sta = pEP0_DataBuf[2] & (1 << 1);   //RTS״̬
							present_sta = pEP0_DataBuf[2] & (1 << 0);   //DTR״̬
							len = 0;
							break;
						}
					default:
						{
							printf("CDC ReqCode%x\r\n", SetupReqCode);
							len = 0xFF;                                       // ʧ
							break;
						}
					}
				}
				/* 豸ϴ */
				else
				{
					switch (SetupReqCode)  // 
					{
					case DEF_GET_LINE_CODING: /* GET_LINE_CODING */
						{
							printf("    GET_LINE_CODING:%d\r\n", pEP0_DataBuf[4]);
							pDescr = pEP0_DataBuf;
							len = sizeof(LINE_CODE);
							((PLINE_CODE)pEP0_DataBuf)->BaudRate = Uart0Para.BaudRate;
							((PLINE_CODE)pEP0_DataBuf)->StopBits = Uart0Para.StopBits;
							((PLINE_CODE)pEP0_DataBuf)->ParityType = Uart0Para.ParityType;
							((PLINE_CODE)pEP0_DataBuf)->DataBits = Uart0Para.DataBits;
							break;
						}
					case DEF_SERIAL_STATE:
						{
							printf("    GET_SERIAL_STATE:%d\r\n", pEP0_DataBuf[4]);
							//SetupLen жܳ
							len = 2;
							//CDCSetSerIdx = 0;
							pEP0_DataBuf[0] = 0;
							pEP0_DataBuf[1] = 0;
							break;
						}
					default:
						{
							printf("    CDC ReqCode%x\r\n", SetupReqCode);
							len = 0xFF;                                       // ʧ
							break;
						}
					}
				}
			}
			else 
			{
				printf("Error %d\r\n", __LINE__);
				len = 0xFF;   /* ʧ */
			}
			
			if (errflag == 0xff) // ֧
			{
				// SetupReqCode = 0xFF;
				R8_UEP0_CTRL = RB_UEP_R_TOG | RB_UEP_T_TOG | UEP_R_RES_STALL | UEP_T_RES_STALL;    // STALL
			}
			else
			{
				if (chtype & 0x80) // ϴ
				{
					len = (SetupReqLen > DevEP0SIZE) ?
						DevEP0SIZE : SetupReqLen;
					SetupReqLen -= len;
				}
				else
				{
					len = 0;        // ´
				}
				R8_UEP0_T_LEN = len;
				R8_UEP0_CTRL  = RB_UEP_R_TOG | RB_UEP_T_TOG | UEP_R_RES_ACK | UEP_T_RES_ACK;    // ĬݰDATA1
			}

			R8_USB_INT_FG = RB_UIF_TRANSFER;
		}
	}
	else if (intflag & RB_UIF_BUS_RST)
	{
		R8_USB_DEV_AD = 0;
		R8_UEP0_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK;
		R8_UEP1_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK | RB_UEP_AUTO_TOG;
		R8_UEP2_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK | RB_UEP_AUTO_TOG;
		R8_UEP3_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK | RB_UEP_AUTO_TOG;
		R8_USB_INT_FG = RB_UIF_BUS_RST;
	}
	else if (intflag & RB_UIF_SUSPEND)
	{
		if (R8_USB_MIS_ST & RB_UMS_SUSPEND)
		{
			;
		}    // 
		else
		{
			;
		}               // 
		R8_USB_INT_FG = RB_UIF_SUSPEND;
	}
	else
	{
		R8_USB_INT_FG = intflag;
	}
}

/*********************************************************************
 * @fn      USB_IRQHandler
 *
 * @brief   USBжϺ
 *
 * @return  none
 */
__attribute__((interrupt("WCH-Interrupt-fast")))
__attribute__((section(".highcode")))
void USB_IRQHandler(void) /* USBжϷ,ʹüĴ1 */
{
	USB_DevTransProcess();
}


/*********************************************************************
*********************************************************************/
